//*****************************************************************************
//*
//*
//*		ObjLib.cpp
//*
//*
//*****************************************************************************
//
//
//	Info von: http://en.wikipedia.org/wiki/Microsoft_Visual_C%2B%2B_Name_Mangling
//
//
//	MangledName ::=
//		'?' OperatorName '@' TypeCode
//		'?' QualifiedName '@' TypeCode
//
//	OperatorName ::=
//		OperatorCode QualifiedName
//
//	QualifiedName ::=
//		Identifier +
//		'?$' Identifier ArgumentList QualifiedName /* template */
//
//	Identifier ::=
//		string '@'
//		'0'-'9' /* reference to previously used identifier */
//
//	OperatorCode ::=
//		'?0'            /* myclass::myclass                                         unary */
//		'?1'            /* myclass::~myclass                                        unary */
//		'?2'            /* myclass::operator new                                          */
//		'?3'            /* myclass::operator delete                                       */
//		'?4'            /* myclass::operator=                                      binary */
//		'?5'            /* myclass::operator>>                                     binary */
//		'?6'            /* myclass::operator<<                                     binary */
//		'?7'            /* myclass::operator!                                       unary */
//		'?8'            /* myclass::operator==                                     binary */
//		'?9'            /* myclass::operator!=                                     binary */
//		'?A'            /* myclass::operator[]                                            */
//		'?B'            /* myclass::(conversion)                              return only */
//		'?C'            /* myclass::operator->                                     binary */
//		'?D'            /* myclass::operator*                                unary/binary */
//		'?E'            /* myclass::operator++                           unary (pre/post) */
//		'?F'            /* myclass::operator--                           unary (pre/post) */
//		'?G'            /* myclass::operator-                                unary/binary */
//		'?H'            /* myclass::operator+                                unary/binary */
//		'?I'            /* myclass::operator&                                unary/binary */
//		'?J'            /* myclass::operator->*                                    binary */
//		'?K'            /* myclass::operator/                                      binary */
//		'?L'            /* myclass::operator%                                      binary */
//		'?M'            /* myclass::operator<                                      binary */
//		'?N'            /* myclass::operator<=                                     binary */
//		'?O'            /* myclass::operator>                                      binary */
//		'?P'            /* myclass::operator>=                                     binary */
//		'?Q'            /* myclass::operator,                                      binary */
//		'?R'            /* myclass::operator()                                            */
//		'?S'            /* myclass::operator~                                       unary */
//		'?T'            /* myclass::operator^                                      binary */
//		'?U'            /* myclass::operator|                                      binary */
//		'?V'            /* myclass::operator&&                                     binary */
//		'?W'            /* myclass::operator||                                     binary */
//		'?X'            /* myclass::operator*=                                     binary */
//		'?Y'            /* myclass::operator+=                                     binary */
//		'?Z'            /* myclass::operator-=                                     binary */
//		'?_0'           /* myclass::myclass/=                                      binary */
//		'?_1'           /* myclass::myclass%=                                      binary */
//		'?_2'           /* myclass::operator>>=                                    binary */
//		'?_3'           /* myclass::operator<<=                                    binary */
//		'?_4'           /* myclass::operator&=                                     binary */
//		'?_5'           /* myclass::operator|=                                     binary */
//		'?_6'           /* myclass::operator^=                                     binary */
//		'?_7'           /* myclass::`vftable'                                             */
//		'?_C'           /* appears to be for string constants                             */
//		'?_F'           /* myclass::`default constructor closure`                         */
//		'?_O'           /* unknown                                                        */
//		'?_R0?AV'       /* myclass::`RTTI Type Descriptor'                                */
//		'?_R1A@?0A@EA@' /* myclass::`RTTI Base Class Descriptor at (0,-1,0,64)'           */
//		'?_R2'          /* myclass::`RTTI Base Class Array'                               */
//		'?_R3'          /* myclass::`RTTI Class Hierarchy Descriptor'                     */
//		'?_R4'          /* myclass::`RTTI Complete Object Locator'                        */
//		'?_U'           /* myclass::operator new[]                                        */
//		'?_V'           /* myclass::operator delete[]                                     */
//
//	TypeCode ::=
//		'A' FunctionType /* private method */
//		'C' FunctionType /* private class method */
//		'E' FunctionType /* private virtual method */
//		'I' FunctionType /* protected method */
//		'K' FunctionType /* protected class method */
//		'M' FunctionType /* protected virtual method */
//		'Q' FunctionType /* public method */
//		'S' FunctionType /* public class method */
//		'U' FunctionType /* public virtual method */
//		'Y' FunctionType /* global function */
//		'2' DataType StorageClass
//		'3' DataType StorageClass
//
//	FunctionType ::=
//		CallingConvention ReturnValue ArgumentList StorageClass
//
//	CallingConvention ::=
//		'A'  /* __cdecl    */
//		'G'  /* __stdcall  */
//		'I'  /* __fastcall */
//		'AA' /* __cdecl for methods ('this' in stack)*/
//		'AE' /* __thiscall ('this' in ECX)*/
//
//	ReturnValue ::=
//		'X'  /* void */
//		'@'  /* void, used with constructors for example */
//		DataType
//
//	ArgumentList ::=
//		'X'  /* void */
//		DataType+
//		DataType+ '@'  /* __cdecl, without vararg */
//		DataType+ 'Z'  /* __cdecl, with vararg */
//
//	DataType ::=
//		SimpleDataType
//		PointerDataType
//
//	SimpleDataType ::=
//		'C'  /* signed char      */
//		'D'  /* char             */
//		'E'  /* unsigned char    */
//		'F'  /* short            */
//		'G'  /* unsigned short   */
//		'H'  /* int              */
//		'I'  /* unsigned int     */
//		'J'  /* long             */
//		'K'  /* unsigned long    */
//		'M'  /* float            */
//		'N'  /* double           */
//		'O'  /* long double      */
//		BackrefableSimpleDataType
//
//	BackrefableSimpleDataType ::=
//		'T'  /* union            */ QualifiedName '@'
//		'U'  /* struct           */ QualifiedName '@'
//		'V'  /* class            */ QualifiedName '@'
//		'W4' /* enum (4 byte)    */ QualifiedName '@'
//		'_J' /* __int64          */
//		'_K' /* unsigned __int64 */
//		'_N' /* bool             */
//		'_W' /* __wchar_t        */
//
//	PointerDataType ::=
//		ReferenceOrPointer Pointer* 'A|B|C' (SimpleDataType | 'X')
//		ReferenceOrPointer Pointer* '6' FunctionType
//		'0'-'9' /* reference to previously used PointerDataType or BackrefableSimpleDataType */
//		ReferenceOrPointer Pointer* '8' FunctionType
//
//	ReferenceOrPointer ::=
//		'A' /* reference to */
//		'P' /* pointer to */
//		'Q' /* const pointer to */
//		'R' /* volatile pointer to */
//
//	Pointer ::=
//		'AP' /* pointer to */
//		'BQ' /* const pointer to */
//		'CR' /* volatile pointer to */
//
//	StorageClass ::
//		'A'  /* Normal Storage     */
//		'B'  /* Volatile Storage   */
//		'C'  /* Const Storage      */
//		'Z'  /* Executable Storage */
//
//
#include	"StdAfx.h"
#include	"ObjLib.h"
#include	<time.h>
#include	<stdio.h>
#include	<stdarg.h>

unsigned	raloc_max=15;
int			cpu_type=0;
int			sec_ok=0;
int			sym_ok=0;
int			funk_ok=0;
int			data_ok=0;
int			extl_ok=0;
int			name_ok=0;
int			funk_glb_ok=1;
int			data_glb_ok=1;
int			extl_glb_ok=1;

static int *options[] =
			{
			&sec_ok,
			&sym_ok,
			&funk_ok,
			&data_ok,
			&extl_ok,
			&name_ok,
			&funk_glb_ok,
			&data_glb_ok,
			&extl_glb_ok,
			};


void InsertFunktion(char *Text);
void InsertSort(int Sort);
void InsertImage(int Nr);
void InsertDown();
void InsertUp();
void InsertInfo(void *Data,int Type,int Offset,int Size);
void InsertTime(const char *Format,long  TimeStamp);
void InsertFormat(const char *Format, ...);


CStringChain	sk_funk(sizeof(int)*4);
CStringChain	sk_data(sizeof(int)*4);
CStringChain	sk_extl(sizeof(int)*4);

int GetProcName(const char *Name,char *Ziel,int Max);

//*****************************************************************************
//*
//*		GetLinkName
//*
//*****************************************************************************
//	Holt einen Namen bis "@@"
int GetLinkName(const char *Name,char *Ziel,int Max)
{
char	temp[256];
int		len=0,x,anz,add=0;



	Ziel[0]=0;

	if(Name[len]=='?')
		{
		len++;
		     if(Name[len]=='0'){add=2;len++;}
		else if(Name[len]=='1'){add=1;len++;strlcpy(Ziel,"~"              ,Max);}
		else if(Name[len]=='2'){add=1;len++;strlcpy(Ziel,"operator new"   ,Max);}
		else if(Name[len]=='3'){add=1;len++;strlcpy(Ziel,"operator delete",Max);}
		else if(Name[len]=='4'){      len++;strlcpy(Ziel,"operator="      ,Max);}
		else if(Name[len]=='5'){      len++;strlcpy(Ziel,"operator>>"     ,Max);}
		else if(Name[len]=='6'){      len++;strlcpy(Ziel,"operator<<"     ,Max);}
		else if(Name[len]=='7'){      len++;strlcpy(Ziel,"operator!"      ,Max);}
		else if(Name[len]=='8'){      len++;strlcpy(Ziel,"operator=="     ,Max);}
		else if(Name[len]=='9'){      len++;strlcpy(Ziel,"operator!="     ,Max);}
		else if(Name[len]=='A'){      len++;strlcpy(Ziel,"operator[]"     ,Max);}
		else if(Name[len]=='B'){      len++;strlcpy(Ziel,"(conversion)"   ,Max);}
		else if(Name[len]=='C'){      len++;strlcpy(Ziel,"operator->"     ,Max);}
		else if(Name[len]=='D'){      len++;strlcpy(Ziel,"operator*"      ,Max);}
		else if(Name[len]=='E'){      len++;strlcpy(Ziel,"operator++"     ,Max);}
		else if(Name[len]=='F'){      len++;strlcpy(Ziel,"operator--"     ,Max);}
		else if(Name[len]=='G'){      len++;strlcpy(Ziel,"operator-"      ,Max);}
		else if(Name[len]=='H'){      len++;strlcpy(Ziel,"operator+"      ,Max);}
		else if(Name[len]=='I'){      len++;strlcpy(Ziel,"operator&"      ,Max);}
		else if(Name[len]=='J'){      len++;strlcpy(Ziel,"operator->*"    ,Max);}
		else if(Name[len]=='K'){      len++;strlcpy(Ziel,"operator/"      ,Max);}
		else if(Name[len]=='L'){      len++;strlcpy(Ziel,"operator%"      ,Max);}
		else if(Name[len]=='M'){      len++;strlcpy(Ziel,"operator<"      ,Max);}
		else if(Name[len]=='N'){      len++;strlcpy(Ziel,"operator<="     ,Max);}
		else if(Name[len]=='O'){      len++;strlcpy(Ziel,"operator>"      ,Max);}
		else if(Name[len]=='P'){      len++;strlcpy(Ziel,"operator>="     ,Max);}
		else if(Name[len]=='Q'){      len++;strlcpy(Ziel,"operator,"      ,Max);}
		else if(Name[len]=='R'){      len++;strlcpy(Ziel,"operator()"     ,Max);}
		else if(Name[len]=='S'){      len++;strlcpy(Ziel,"operator~"      ,Max);}
		else if(Name[len]=='T'){      len++;strlcpy(Ziel,"operator^"      ,Max);}
		else if(Name[len]=='U'){      len++;strlcpy(Ziel,"operator|"      ,Max);}
		else if(Name[len]=='V'){      len++;strlcpy(Ziel,"operator&&"     ,Max);}
		else if(Name[len]=='W'){      len++;strlcpy(Ziel,"operator||"     ,Max);}
		else if(Name[len]=='X'){      len++;strlcpy(Ziel,"operator*="     ,Max);}
		else if(Name[len]=='Y'){      len++;strlcpy(Ziel,"operator+="     ,Max);}
		else if(Name[len]=='Z'){      len++;strlcpy(Ziel,"operator-="     ,Max);}
		else if(Name[len]=='_')
			{
			len++;

			     if(Name[len]=='0'){  len++;strlcpy(Ziel,"myclass/="      ,Max);}
			else if(Name[len]=='1'){  len++;strlcpy(Ziel,"myclass%="      ,Max);}
			else if(Name[len]=='2'){  len++;strlcpy(Ziel,"operator>>="    ,Max);}
			else if(Name[len]=='3'){  len++;strlcpy(Ziel,"operator<<="    ,Max);}
			else if(Name[len]=='4'){  len++;strlcpy(Ziel,"operator&="     ,Max);}
			else if(Name[len]=='5'){  len++;strlcpy(Ziel,"operator|="     ,Max);}
			else if(Name[len]=='6'){  len++;strlcpy(Ziel,"operator^="     ,Max);}
			else if(Name[len]=='7'){  len++;strlcpy(Ziel,"vftable"        ,Max);}
			else if(Name[len]=='C'){  len++;strlcpy(Ziel,"strconst "      ,Max);}
			else if(Name[len]=='U'){  len++;strlcpy(Ziel,"new[] "         ,Max);}
			else if(Name[len]=='V'){  len++;strlcpy(Ziel,"delete[]"       ,Max);}
			else if(Name[len]=='G'){  len++;strlcpy(Ziel,"??? "           ,Max);}
			}
		}

	while(Name[len]!='@' || Name[len+1]!='@')
		{
		if(Name[len]== 0 )break;
		if(Name[len]=='@')len++;

		x=anz=strchar(Name+len,'@');


		if(x>sizeof(temp)-1)x=sizeof(temp)-1;
		strlcpy(temp,Name+len,x+1);
		len+=anz;

		if(Ziel[0])strlpre(Ziel,"::",Max);
				   strlpre(Ziel,temp,Max);


		}

	if(Name[len]=='@')len++;
	if(Name[len]=='@')len++;

	if(add)
		{
		x=strcspn(Ziel,":~");
		if(x>=sizeof(temp))x=sizeof(temp)-1;
		memcpy(temp,Ziel,x);temp[x]=0;
		if(add==2)strlcat(Ziel,"::",Max);
		strlcat(Ziel,temp,Max);
		}


return len;
}

//*****************************************************************************
//*
//*		GetVarName
//*
//*****************************************************************************
int GetVarName(const char *Name,char *Ziel,int Max)
{
char	temp[256];
int		len=0,const_fl=0;



	Ziel[0]=0;

	while(Name[len])
		{
		if(Name[len]=='@')			// leer Variabel
			{
			len++;
			break;
			}
		if(Name[len]=='P' || Name[len]=='?')
			{
			len++;
			switch(Name[len])
				{
			case 'A':	break;
			case 'B':	const_fl=1;						break;
			case '6':	len++;
						len+=GetProcName(Name+len,Ziel,Max);
						return len;
			default:	strlpre(Ziel,"??? "   ,Max);	break;
				}
			len++;
			strlcat(Ziel,"*" ,Max);
			continue;
			}

		if(Name[len]=='A')
			{
			len++;
			switch(Name[len])
				{
			case 'A':	break;
			case 'B':	const_fl=1;						break;
			default:	strlpre(Ziel,"??? "   ,Max);	break;
				}
			len++;
			strlcat(Ziel,"&" ,Max);
			continue;
			}

		if(Name[len]=='Q')
			{
			len++;
			switch(Name[len])
				{
			case 'A':	break;
			case 'B':	const_fl=1;						break;
			default:	strlpre(Ziel,"??? "   ,Max);	break;
				}
			len++;
			if(Name[len]>='0' && Name[len]<='9')
				{
				len+=strchar(Name+len,'$');
				if(Name[len]=='$')len++;
				if(Name[len]=='$')len++;
				}
			strlcat(Ziel,"[]" ,Max);
			continue;
			}

		switch(Name[len])
			{
		case 'C':	 strlpre(Ziel,"signed char "	,Max);	len++;break;
		case 'D':	 strlpre(Ziel,"char "			,Max);	len++;break;
		case 'E':	 strlpre(Ziel,"unsigned char "  ,Max);	len++;break;
		case 'F':	 strlpre(Ziel,"short "   		,Max);	len++;break;
		case 'G':	 strlpre(Ziel,"unsigned short " ,Max);	len++;break;
		case 'H':	 strlpre(Ziel,"int "   			,Max);	len++;break;
		case 'I':	 strlpre(Ziel,"unsigned int "   ,Max);	len++;break;
		case 'J':	 strlpre(Ziel,"long "   		,Max);	len++;break;
		case 'K':	 strlpre(Ziel,"unsigned long "	,Max);	len++;break;
		case 'M':	 strlpre(Ziel,"float "   		,Max);	len++;break;
		case 'N':	 strlpre(Ziel,"double "   		,Max);	len++;break;
		case 'O':	 strlpre(Ziel,"long double "   	,Max);	len++;break;
		case 'X':	 strlpre(Ziel,"void "			,Max);	len++;break;
		case 'V':	 len++;											// class
					 if(Name[len]=='1' || Name[len]=='0')
						{
						len++;
						strlpre(Ziel,"this ",Max);
						if(Name[len]=='@')len++;
						}
					 else
						{
						len+=GetLinkName(Name+len,temp,sizeof(temp));
						strlpre(Ziel,temp,Max);
						strlpre(Ziel," ",Max);
						}

					 break;
		case 'Z':	 len++;
					 if(Name[len]=='Z')strlpre(Ziel,"...",Max);
					 break;
		case '_':	 len++;
					 if(Name[len]=='J'){strlpre(Ziel,"__int64 "  ,Max);len++;break;}
					 if(Name[len]=='N'){strlpre(Ziel,"bool "	 ,Max);len++;break;}
					 if(Name[len]=='K'){strlpre(Ziel,"__uint64 " ,Max);len++;break;}
					 if(Name[len]=='W'){strlpre(Ziel,"__wchar_t ",Max);len++;break;}
					 strlpre(Ziel,"??? ",Max);
					 break;
		case 'W':	 if(Name[len+1]!='4')break;						// enum
					 len++;
		case 'T':													// union
		case 'U':	 len++;											// struct
					 len+=GetLinkName(Name+len,temp,sizeof(temp));
					 strlcat(temp," ",sizeof(temp));
					 strlpre(Ziel,temp,Max);
					 break;

					len++;
					 len+=GetLinkName(Name+len,temp,sizeof(temp));
					 strlcat(temp," ",sizeof(temp));
					 strlpre(Ziel,temp,Max);
					 break;
		default:	 strlpre(Ziel,"??? ",Max);
					 len++;
					 break;
			}

		break;
		}

	if(const_fl)strlpre(Ziel,"const ",Max);

return len;
}

//*****************************************************************************
//*
//*		GetProcName
//*
//*****************************************************************************
int GetProcName(const char *Name,char *Ziel,int Max)
{
char	temp[256];
int		len=0,nr=0,const_fl=0;


	switch(*Name)
		{
	default:	strlcpy(Ziel,"(???*)("       ,Max);			break;
	case 'E':	strlcpy(Ziel,"(_thiscall*)(" ,Max);	len++;	break;
	case 'I':	strlcpy(Ziel,"(_fastcall*)(" ,Max);	len++;	break;
	case 'G':	strlcpy(Ziel,"(_stdcall*)("  ,Max);	len++;	break;
	case 'A':	if(Name[1]!='A')
					{
					strlcpy(Ziel,"(_cdecl*)(",Max);	len++;	break;
					}

				len++;
				strlcpy(Ziel,"(_thiscall*)(",Max);	len++;	break;
		}



	len+=GetVarName(Name+len,temp,sizeof(temp));
	strlpre(Ziel,temp,Max);
	strcpy(temp,"void* ");

	while(Name[len])
		{
		if(Name[len]=='Z'){len++;break;}
		if(Name[len]=='@'){len++;continue;}
		if(Name[len]=='1'){len++;continue;}
		if(nr)
			{
			strlcat(Ziel,",",Max);
			}
		if(Name[len]=='0')
			{
			len++;
			}
		else{
			len+=GetVarName(Name+len,temp,sizeof(temp));
			}

		nr++;
		strlcat(Ziel,temp,Max);
		}

	strlcat(Ziel,") ",Max);


return len;
}

//*****************************************************************************
//*
//*		LinknameToCppname
//*
//*****************************************************************************
//	Wandelt einen Linkernamen in einen Funktionsnamen um
const char *LinknameToCppname(const char *LinkName)
{
static char		ziel[1024];
char			para[1024];
char			temp [256];
char			flags[256];
char		   *name=(char*)LinkName;
int				const_fl,nr,i;

/* Windows Version
typedef	(__stdcall *XXX)(const char *,char *,int, int);
static	HMODULE	h;
static	XXX		x;


	if(!h)
		{
		h=LoadLibrary("dbghelp.dll");
		x=(XXX)GetProcAddress(h,"UnDecorateSymbolName");
		}

	char ccc[1024];
	x(LinkName,ccc,1024,0);
*/


	temp [0] = 0;
	flags[0] = 0;

	if(name[0]=='_')
		{
		if(name[1]!=0)name++;
		return name;
		}

	if(name[0]!='?')
		{
		return name;
		}

	ziel[0]=0;
	name++;

	name+=GetLinkName(name,ziel,sizeof(ziel));
	const_fl=0;


	switch(*name)
		{
	case 'Q':	strlcat(flags,"public "           ,sizeof(flags));	const_fl=1;	break;
	case 'I':	strlcat(flags,"protected "        ,sizeof(flags));	const_fl=1;	break;
	case 'A':	strlcat(flags,"private "          ,sizeof(flags));	const_fl=1;	break;

	case 'U':	strlcat(flags,"virtual public "   ,sizeof(flags));	const_fl=1;	break;
	case 'M':	strlcat(flags,"virtual protected ",sizeof(flags));	const_fl=1;	break;
	case 'E':	strlcat(flags,"virtual private "  ,sizeof(flags));	const_fl=1;	break;

	case 'S':	strlcat(flags,"static public "    ,sizeof(flags));	            break;
	case 'K':	strlcat(flags,"static protected " ,sizeof(flags));	            break;
	case 'C':	strlcat(flags,"static private "   ,sizeof(flags));	            break;

	case 'Y':	break;
	case '6':	const_fl=1;	break;
	case '1':
	case '2':
	case '3':	name++;
				name+=GetVarName(name,temp,sizeof(temp));
				switch(*name)
					{
				case  0 :	       break;
				case 'A':	name++;break;
				case 'B':	strlpre(temp,"const ",sizeof(temp));name++;break;
				default:	return LinkName;
					}
				if(*name)
					return LinkName;
				strlpre(ziel,temp,sizeof(ziel));
				return ziel;

	default:	return LinkName;
		}


	name++;

	if(const_fl)
		{
		switch(*name)
			{
		case 'A':strlcat(flags,"const ",sizeof(flags));	break;
		case 'B':break;
		default: return LinkName;
			}
		const_fl;
		name++;
		}

	switch(*name)
		{
	case 'E':	strlcat(flags,"_thiscall ",sizeof(flags));	break;
	case 'A':	strlcat(flags,"_cdecl "   ,sizeof(flags));	break;
	case 'G':	strlcat(flags,"_stdcall " ,sizeof(flags));	break;
	case 'I':	strlcat(flags,"_fastcall ",sizeof(flags));	break;
	default:	return LinkName;
		}

	name++;
	if(*name)
		{
		name+=GetVarName(name,temp,sizeof(temp));
		strlpre(ziel,temp,sizeof(ziel));
		}

	strlcat(ziel,"(",sizeof(ziel));
	strcpy(temp,"void* ");
	nr=0;

	while(*name)
		{
		if(*name=='Z')break;
		if(*name=='@'){name++;continue;}

		if(*name>='0' && *name<='9')
			{
			i=*name-'0';

			if(i>=nr)
				{
				strcpy(temp,"???");
				}
			else{
				strcpy(temp,para+i*100);
				}

			if(nr<10)strcpy (para+nr*100,temp);
			if(nr   )strlcat(ziel,"," ,sizeof(ziel));
			         strlcat(ziel,temp,sizeof(ziel));

			nr++;
			name++;
			continue;
			}

		if(nr)
			{
			strlcat(ziel,",",sizeof(ziel));
			}

		name+=GetVarName(name,temp,sizeof(temp));

		if(nr<10)
			{
			strlcpy(para+nr*100,temp,128);
			}

		strlcat(ziel,temp,sizeof(ziel));
		nr++;
		}

	strlcat(ziel,") " ,sizeof(ziel));
	strlcat(ziel,flags,sizeof(ziel));



return ziel;
}


//*****************************************************************************
//*
//*		LoadInit
//*
//*****************************************************************************
void LoadInit()
{

	sk_funk.Del();
	sk_data.Del();
	sk_extl.Del();

}

//*****************************************************************************
//*
//*		LoadObj
//*
//*****************************************************************************
int LoadObj(const char *Ptr,unsigned int Size,unsigned Offset)
{
ObjHeader	*header;
SymEntry	*symbols,*sym;
SecEntry	*section,*sec;
RealocEntry	*rel;
const char  *symnames,*name;
unsigned	 data[4];
unsigned	 namepos,namelen,secpos,seclen,i,j,k,pos=0;
unsigned	 funk_count=0,data_count=0,extl_count=0;
char		 temp[16];



	header = (ObjHeader*)Ptr;

	if(Size<sizeof(ObjHeader))return 1;

	InsertInfo(header,OBJ_TYPE_OBJECT,Offset,Size);

	data[2]=Offset;

	InsertFormat("Datei-Gre = %i",Size);
	InsertFormat("Datei-Offest = %08Xh (%i)",Offset,Offset);

	switch(header->cpu_type)
		{
	case 0x014C:	cpu_type= 1;	InsertFormat("CPU-Type = i386");		break;
	case 0x014D:	cpu_type= 2;	InsertFormat("CPU-Type = i486");		break;
	case 0x014E:	cpu_type= 3;	InsertFormat("CPU-Type = Pentium");		break;
	case 0x0162:	cpu_type= 4;	InsertFormat("CPU-Type = MIPS R3000");	break;
	case 0x0166:	cpu_type= 5;	InsertFormat("CPU-Type = MIPS R4000");	break;
	case 0x0184:	cpu_type= 6;	InsertFormat("CPU-Type = DEC Alpha");	break;
	case 0x01F0:	cpu_type= 7;	InsertFormat("CPU-Type = PowerPC");		break;
	case 0x457F:	cpu_type= 8;	InsertFormat("CPU-Type = ARM");			break;
	case 0x8664:	cpu_type= 9;	InsertFormat("CPU-Type = Intel X64");	break;
	default:		cpu_type= 0;	InsertFormat("CPU-Type = Unbekannt");	return 2;
		}


	InsertTime("Zeitstempel = ",header->time_stamp);
	InsertFormat("Abschnitt-Offest = %08Xh (%i)",header->sym_addr+Offset,header->sym_addr+Offset);
	InsertFormat("Abschnitt-Anzahl = %i",header->sec_count);

	if(Size<sizeof(ObjHeader)+header->opt_size)return 1;


//******************** Abschnitte *********************************************

	secpos = header->opt_size +sizeof(ObjHeader);
	seclen = header->sec_count*sizeof(SecEntry);
	if(Size<secpos+seclen)return 1;
	section=(SecEntry*)(Ptr+secpos);

if(sec_ok)
	{
	InsertImage(1);
	InsertFormat("Abschnitte");
	InsertUp();
	InsertSort(1);


	for(i=0;i<header->sec_count;i++)
		{
		sec=section+i;
		InsertInfo(sec,OBJ_TYPE_SECTION,(char*)sec-Ptr+Offset,sizeof(*sec));
		memcpy(temp,sec->name,8);
		temp[8]=0;
		InsertImage(7);
		InsertFormat("%s (%i)",temp,i);
		InsertUp();
		InsertFormat("Lnge = %i",sec->raw_size);
		InsertFormat("Datei-Offset = %08Xh (%i)",sec->raw_attr+Offset,sec->raw_attr+Offset);
		InsertFormat("Realokierungen = %i",sec->rel_count);
		InsertFormat("Zeilennummern = %i" ,sec->line_count);
		if(sec->flags&0x00000008)InsertFormat("No PAD");
		if(sec->flags&0x00000020)InsertFormat("Enthlt Programmcode");
		if(sec->flags&0x00000040)InsertFormat("Enthlt initialisierte Daten");
		if(sec->flags&0x00000080)InsertFormat("Enthlt nicht initialisierte Daten");
		if(sec->flags&0x00000200)InsertFormat("Enthlt Zusatzinfos (Kommentare)");
		if(sec->flags&0x00000800)InsertFormat("Nicht im Speicher abbilten");
		if(sec->flags&0x00001000)InsertFormat("Kompiniere Segmente");
		if(sec->flags&0x01000000)InsertFormat("Enthlt Reallokationsdaten");
		if(sec->flags&0x02000000)InsertFormat("Inhalt muss nicht gesichert werden");
		if(sec->flags&0x04000000)InsertFormat("Inhalt nicht cachen");
		if(sec->flags&0x08000000)InsertFormat("Inhalt nicht auslagern");
		if(sec->flags&0x10000000)InsertFormat("Nicht von mehreren CPU's ausfhren");
		if(sec->flags&0x20000000)InsertFormat("Ausfhrbar");
		if(sec->flags&0x40000000)InsertFormat("Lesbar");
		if(sec->flags&0x80000000)InsertFormat("Schreibgeschtzt");

		if(sec->rel_count)
			{
			InsertFormat("Reallokationsdaten");
			InsertUp();
			if(Size<sec->rel_addr+sec->rel_count*sizeof(RealocEntry))return 1;
			rel=(RealocEntry*)(Ptr+sec->rel_addr);
			for(j=0;j<sec->rel_count;j++,rel++)
				{
				InsertFormat("Virtuele Adresse = %08Xh",rel->virt_addr);
				InsertFormat("Symbolnummer = %i",rel->sym_nr);
				switch(rel->type)
					{
				case  6: InsertFormat("Typ = DIR32");			break;
				case  7: InsertFormat("Typ = RVA32");			break;
				case 20: InsertFormat("Typ = REL32");			break;
				default: InsertFormat("Typ = %i",rel->type);	break;
					}
				}
			InsertDown();
			}

		InsertDown();
		}

	InsertDown();
	}

//******************** Symbole ************************************************

	symbols = (SymEntry*)(Ptr+header->sym_addr);
	namepos = header->sym_addr+sizeof(SymEntry)*header->sym_count;

	if(Size<namepos+4)return 1;
	symnames = Ptr+namepos;
	namelen  = *(unsigned*)symnames;
	if(Size<namepos+namelen)return 1;
	if(symnames[namelen-1] )return 3;

if(sym_ok)
	{
	InsertImage(2);
	InsertFormat("Symbole");
	InsertUp();
	InsertSort(1);

	for(i=0;i<header->sym_count;i++)
		{
		sym=symbols+i;

		InsertInfo(sym,OBJ_TYPE_SYMBOL,(char*)sym-Ptr+Offset,sizeof(*sym));

		if(sym->offset[0])
			{
			memcpy(temp,sym->name,8);
			temp[8]=0;
			name=temp;
			}
		else{
			if(sym->offset[1]>=namelen)return 1;
			name=symnames+sym->offset[1];
			}

		InsertImage(7);
		InsertFormat("%s (%i)",name,i);

		if(sym->sec_nr>0 && sym->sec_nr<=header->sec_count)
			{
			sec=section+sym->sec_nr-1;
			memcpy(temp,sec->name,8);
			temp[8]=0;
			}
		else{
			strcpy(temp,"???");
			}

		InsertUp();
		InsertFormat("Abschnitt = %i ('%s')",sym->sec_nr-1,temp);
		InsertFormat("Value = %i",sym->value);
		InsertFormat("Type = %i",sym->type);
		switch(sym->art)
			{
		case   2: InsertFormat("Art = Extern");			break;
		case   3: InsertFormat("Art = Static");			break;
		case   6: InsertFormat("Art = Label");			break;
		case 101: InsertFormat("Art = Funktion");		break;
		case 103: InsertFormat("Art = File : '%s'",(sym->ext_nr)? sym+1:0);	break;
		case 104: InsertFormat("Art = Sektion");		break;
		case 105: InsertFormat("Art = Klasscall");		break;
		default:  InsertFormat("Art = %i",sym->art);	break;
			}

		InsertFormat("Zusatz Sektionen = %i",sym->ext_nr);
		InsertDown();
		i+=sym->ext_nr;
		}

	InsertDown();
	}

//******************** Funktionen *********************************************

if(funk_ok || funk_glb_ok)
	{
	for(i=0;i<header->sym_count;i++)
		{
		sym=symbols+i;
		if(sym->type!=32)continue;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr<=0)continue;
		if(sym->sec_nr>header->sec_count)continue;
			sec=section+sym->sec_nr-1;
		if(~sec->flags&0x20000000)continue;
		funk_count++;
		}

	if(funk_ok)
		{
		InsertImage(3);
		InsertFormat("Funktionen = %i",funk_count);
		InsertUp();
		InsertSort(1);
		}

	for(i=0;i<header->sym_count;i++)
		{
		sym=symbols+i;
		if(sym->type!=32)continue;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr<=0)continue;
		if(sym->sec_nr>header->sec_count)continue;
			sec=section+sym->sec_nr-1;
		if(~sec->flags&0x20000000)continue;	// ausfhrbar


		if(sym->offset[0])
			{
			memcpy(temp,sym->name,8);
			temp[8]=0;
			name=temp;
			}
		else{
			if(sym->offset[1]>=namelen)return 1;
			name=symnames+sym->offset[1];
			}

		name=LinknameToCppname(name);
		pos=sec->raw_attr+sym->value+Offset;

		if(funk_glb_ok)
			{
			data[0]=pos;
			data[1]=sec->raw_size-sym->value;
			data[3]=sym->sec_nr-1;
			sk_funk.Add(name,CSC_END,data);
			}

		if(!funk_ok)continue;

		InsertImage(7);
		InsertFormat("%s",name);
		InsertUp();

		InsertFormat("Lnge = %i",sec->raw_size-sym->value);
		InsertFormat("Datei-Offset = %08Xh (%i)",pos,pos);
		InsertFormat("Realokierungen = %i",sec->rel_count);
		InsertUp();
		InsertSort(1);

		pos=sym->value;

		rel=(RealocEntry*)(Ptr+sec->rel_addr);
 		for(j=0,k=0;j<sec->rel_count;j++,rel++)
			{
			if(k>=raloc_max)
				{
				InsertFormat("usw. ...");
				break;
				}
			if(pos>rel->virt_addr)continue;
			if(rel->sym_nr<0 || rel->sym_nr>=header->sym_count)
				{
				InsertFormat("??? (%i/%08Xh)",rel->sym_nr,rel->virt_addr-pos);
				continue;
				}

			   sym=symbols+rel->sym_nr;
			if(sym->offset[0])
				{
				memcpy(temp,sym->name,8);
				temp[8]=0;
				name=temp;
				}
			else{
				if(sym->offset[1]>=namelen)return 1;
				name=symnames+sym->offset[1];
				}

			name=LinknameToCppname(name);
			InsertFormat("%s (%i/%08Xh)",name,rel->sym_nr,rel->virt_addr-pos);
			k++;
			}

		InsertDown();
		InsertDown();
		}

	if(funk_ok)InsertDown();
	}

//******************** Daten **************************************************

if(data_ok || data_glb_ok)
	{
	for(i=0;i<header->sym_count;i++)
		{
		   sym=symbols+i;
		if(sym->type!=0)continue;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr<=0)continue;
		if(sym->sec_nr>header->sec_count)continue;
			sec=section+sym->sec_nr-1;
		if(sec->flags&0x20000000)continue;
		data_count++;
		}

	if(data_ok)
		{
		InsertImage(4);
		InsertFormat("Daten = %i",data_count);
		InsertUp();
		InsertSort(1);
		}

	for(i=0;i<header->sym_count;i++)
		{
		sym=symbols+i;
		if(sym->type!=0)continue;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr<=0)continue;
		if(sym->sec_nr>header->sec_count)continue;
			sec=section+sym->sec_nr-1;
		if(sec->flags&0x20000000)continue;	// nicht ausfhrbar


		if(sym->offset[0])
			{
			memcpy(temp,sym->name,8);
			temp[8]=0;
			name=temp;
			}
		else{
			if(sym->offset[1]>=namelen)return 1;
			name=symnames+sym->offset[1];
			}

		name=LinknameToCppname(name);

		if(data_glb_ok)
			{
			data[0]=pos;
			data[1]=sec->raw_size-sym->value;
			data[3]=sym->sec_nr-1;
			sk_data.Add(name,CSC_END,data);
			}

		if(!data_ok)continue;

		InsertImage(7);
		InsertFormat("%s",name);
		InsertUp();

		pos=sec->raw_attr+sym->value+Offset;

		if(sec->flags&0x00000040)
				InsertFormat("Datei-Offset = %08Xh (%i)",pos,pos);
		else	InsertFormat("Datei-Offset = %08Xh (%i) auf 0 gesetzt ",sym->value,sym->value);

		pos=sym->value;

		InsertFormat("Realokierungen = %i",sec->rel_count);
		InsertUp();
		InsertSort(1);

		rel=(RealocEntry*)(Ptr+sec->rel_addr);
 		for(j=0,k=0;j<sec->rel_count;j++,rel++)
			{
			if(k>=raloc_max)
				{
				InsertFormat("usw. ...");
				break;
				}
			if(pos>rel->virt_addr)continue;

			if(rel->sym_nr<0 || rel->sym_nr>=header->sym_count)
				{
				InsertFormat("??? (%i/%08Xh)",rel->sym_nr,rel->virt_addr-pos);
				continue;
				}

			   sym=symbols+rel->sym_nr;
			if(sym->offset[0])
				{
				memcpy(temp,sym->name,8);
				temp[8]=0;
				name=temp;
				}
			else{
				if(sym->offset[1]>=namelen)return 1;
				name=symnames+sym->offset[1];
				}

			name=LinknameToCppname(name);
			InsertFormat("%s (%i/%08Xh)",name,rel->sym_nr,rel->virt_addr-pos);
			k++;
			}

		InsertDown();
		InsertDown();
		}

	if(data_ok)InsertDown();
	}

//******************** Externe Links ******************************************

if(extl_ok || extl_glb_ok)
	{
	for(i=0;i<header->sym_count;i++)
		{
		   sym=symbols+i;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr>0)continue;

		extl_count++;
		}

	if(extl_ok)
		{
		InsertImage(5);
		InsertFormat("Externe Links = %i",extl_count);
		InsertUp();
		InsertSort(1);
		}

	for(i=0;i<header->sym_count;i++)
		{
		sym=symbols+i;
		switch(sym->art)
			{
		case 2:		break;
		default:	continue;
			}

		if(sym->sec_nr>0)continue;

		if(sym->offset[0])
			{
			memcpy(temp,sym->name,8);
			temp[8]=0;
			name=temp;
			}
		else{
			if(sym->offset[1]>=namelen)return 1;
			name=symnames+sym->offset[1];
			}

		name=LinknameToCppname(name);

		if(extl_glb_ok)
			{
			data[0]=pos;
			data[1]=sec->raw_size-sym->value;
			data[3]=sym->sec_nr-1;
			sk_extl.Add(name,CSC_END,data);
			}

		if(!extl_ok)continue;

		InsertImage(7);
		InsertFormat("%s",name);
		}

	if(extl_ok)InsertDown();
	}


return 0;
}



//*****************************************************************************
//*
//*		LoadLib
//*
//*****************************************************************************
int LoadLib(const char *Ptr,unsigned int Size,unsigned Offset)
{
LibHeader	*lib;
unsigned	*ptr;
unsigned	 size;
unsigned	 pos;
unsigned	 off;
unsigned	 val;
unsigned	 end;
unsigned	 flen=0;
unsigned	 count;
unsigned	 wo,len,x;
char		 temp[256];
char		*files=0,*name;


	cpu_type = 0;


	if(Size<8+60)return 11;
	if(memcmp(Ptr,"!<arch>\n",8))return 12;


	lib=(LibHeader*)(Ptr+8);
	if(memcmp(lib->type,"/               ",16))return 12;
	if(memcmp(lib->id,"\x60\x0A"          , 2))return 12;

	pos = 8+60;

	if(name_ok && Size>8+60+4)			// Bibliotheksnamen
		{
		off   = 8+60;
		ptr   = (unsigned*)(Ptr+off);
		count = btol_dw(*ptr);
		ptr++;

		off  += 4+count*4;

		if(count>0)
			{
			InsertImage(10);
			InsertFormat(" Namen = %u",count);
			InsertUp();
										// Namen dekodieren
			for(;count && off+2<Size;count--)
				{
				size = strnlen(Ptr+off,Size-off);
				val  = btol_dw(*ptr);

				strlcpy(temp,Ptr+val,17,Size-val);
				end  = strcut(temp);

				if(end>0 && temp[end-1]=='/')
					{
					temp[end-1]=0;
					}

				InsertFormat("%s",LinknameToCppname(Ptr+off));
				InsertUp();
				InsertFormat("%s",temp);
				InsertDown();

				off += size+1;
				ptr += 1;
				}

			InsertDown();
			}
		}


	strlcpy(temp,lib->size,10);
	size= atoi(temp);

	pos+=size;
	if(pos&1)pos++;
	if(pos+60>=Size)return 12;

	InsertSort(1);

	while(pos<Size)						// Lade Objekt Dateien
		{
		lib=(LibHeader*)(Ptr+pos);
		if(memcmp(lib->id,"\x60\x0A", 2))return 12;

		if(pos+60>=Size)return 12;

		strlcpy(temp,lib->size,10);
		size= atoi(temp);


		if(!memcmp(lib->type,"/               ",16))
			{
			strlcpy(temp,lib->size,10);
			size= atoi(temp);

			pos+=size+60;
			if(pos&1)pos++;
			if(pos+60>=Size)return 12;
			continue;
			}

		if(!memcmp(lib->type,"//",2))	// Liste mit File-Namen
			{
			strlcpy(temp,lib->size,10);
			flen  = atoi(temp);
			files = (char*)Ptr+pos+60;
			if(flen>0 && (files[flen-1]!=0 && files[flen-1]!='\n'))return 12;

			pos+=flen+60;
			if(pos&1)pos++;
			if(pos+60>=Size)return 12;

			continue;
			}

		if(lib->type[0]=='/')			// Obj Name
			{
			if(lib->type[1]< '0')return 12;
			if(lib->type[1]> '9')return 12;

			strlcpy(temp,lib->type+1,15);
			wo = atoi(temp);
			if(wo>=flen)return 12;
			name=files+wo;
			}							// Obj Name ber Index
		else if(lib->type[0]>='0' && lib->type[0]<='9')
			{
			memcpy(temp,lib->type,16);
			temp[16]=0;
			wo=strcut(temp);
			if(wo<=0 || temp[wo-1]!='/')return 14;
			wo--;
			temp[wo]=0;
			name=temp;

			wo=strwlen(temp,0x01);
			if(!stricmp(temp+wo,".OBJ") || !stricmp(temp+wo,".O"))
				{
				wo=atoi(temp)-1;
				for(x=0;wo>0 && x<flen;wo--)
					{
					x+=strlen(files+x)+1;
					}
				if(wo==0 && x<flen)name=files+x;
				}
			}
		else{						  // Obj Name aus Index
			memcpy(temp,lib->type,16);
			temp[16]=0;
			wo=strcut(temp);
			if(wo<=0 || temp[wo-1]!='/')return 14;
			temp[wo-1]=0;
			name=temp;
			}

		pos+=60;
		if(pos+size>Size)return 12;

		if(Size>=4 && *(DWORD*)(Ptr+pos)==0xFFFF0000)
			{
			if(LoadDll(Ptr+pos,size,pos))return 13;
			pos+=size;
			if(pos&1)pos++;
			continue;
			}


		len = strcwlen(name,0x40,sizeof(temp)-1);

		if(name[len])
			{
			if(len>0 && name[len-1]=='\\')len--;
			memcpy(temp,name,len);
			temp[len]=0;
			name=temp;
			}


		InsertImage(6);
		InsertFormat("%s",name);
		InsertUp();
		if(LoadObj(Ptr+pos,size,pos))return 13;
		InsertDown();

		pos+=size;
		if(pos&1)pos++;
		}

	InsertDown();

return 0;
}


//*****************************************************************************
//*
//*		LoadRel
//*
//*****************************************************************************
int  LoadRel(const char *Ptr,unsigned int Size,unsigned *Data)
{
ObjHeader	*header;
SymEntry	*symbols,*sym;
SecEntry	*section,*sec;
RealocEntry	*rel;
const char  *symnames,*name;
unsigned	 offset,len;
unsigned	 namepos,namelen,secpos,seclen,j,pos;
unsigned	 funk_count=0,data_count=0,extl_count=0;
char		 temp[16];


	   offset = Data[2];
	if(offset>=Size)return 1;

	Ptr  += offset;
	Size -= offset;
	pos   =	Data[0]-offset;
	len   = Data[1];

	header = (ObjHeader*)Ptr;

	if(Size<sizeof(ObjHeader))return 1;
	if(Size<sizeof(ObjHeader)+header->opt_size)return 1;

	switch(header->cpu_type)
		{
	case 0x014C:	break;
	case 0x014D:	break;
	case 0x014E:	break;
	case 0x0162:	break;
	case 0x0166:	break;
	case 0x0184:	break;
	case 0x01F0:	break;
	case 0x457F:	break;
	case 0x8664:	break;
	default:		return 2;
		}

//******************** Abschnitte *********************************************

	secpos = header->opt_size +sizeof(ObjHeader);
	seclen = header->sec_count*sizeof(SecEntry);
	if(Size<secpos+seclen)return 1;
	section=(SecEntry*)(Ptr+secpos);


//******************** Symbole ************************************************

	symbols = (SymEntry*)(Ptr+header->sym_addr);
	namepos = header->sym_addr+sizeof(SymEntry)*header->sym_count;

	if(Size<namepos+4)return 1;
	symnames = Ptr+namepos;
	namelen  = *(unsigned*)symnames;
	if(Size<namepos+namelen)return 1;
	if(symnames[namelen-1] )return 3;

//*****************************************************************************

	if(Data[3]>=header->sec_count)return 4;

 	sec=section+=Data[3];
	pos-=sec->raw_attr;

	if(sec->rel_count)
		{
		InsertFormat("Reallokationsdaten");
		InsertUp();

		if(Size<sec->rel_addr+sec->rel_count*sizeof(RealocEntry))return 1;

		rel=(RealocEntry*)(Ptr+sec->rel_addr);

		for(j=0;j<sec->rel_count;j++,rel++)
			{
			if(pos    > rel->virt_addr)continue;
			if(pos+len<=rel->virt_addr)continue;

			   sym=symbols+rel->sym_nr;
			if(sym->offset[0])
				{
				memcpy(temp,sym->name,8);
				temp[8]=0;
				name=temp;
				}
			else{
				if(sym->offset[1]>=namelen)return 1;
				name=symnames+sym->offset[1];
				}

			name=LinknameToCppname(name);

			InsertFormat("%s (%08Xh)",name,rel->virt_addr-pos);
			}
		InsertDown();
		}

return 0;
}


//*****************************************************************************
//*
//*		LoadDll
//*
//*****************************************************************************
int LoadDll(const char *Ptr,unsigned int Size,unsigned Offset)
{
ObjHeader	*header;
const char  *name;
unsigned	 data[4];
unsigned	 i,j;


 	cpu_type = 0;


	header = (ObjHeader*)Ptr;

	if(Size<sizeof(ObjHeader))return 1;


	data[2]=Offset;

	name=Ptr+sizeof(ObjHeader);
	i=strnlen(name,Size-sizeof(ObjHeader));
	if(Size<=sizeof(ObjHeader)+i)return 1;
	j=strnlen(name+i+1,Size-sizeof(ObjHeader));
	if(Size<=sizeof(ObjHeader)+i+j+1)return 1;

	name=LinknameToCppname(name);

	if(funk_glb_ok)
		{
		data[0]=0;
		data[1]=0;
		data[3]=0;
		sk_funk.Add(name,CSC_END,data);
		}


return 0;
}

//*****************************************************************************
//*
//*		LoadOptions
//*
//*****************************************************************************
//	Ladeoptionen einstellen
//	Item	: Art der Option (0=alle)
//				1 = sec_ok
//				2 = sym_ok
//				3 = funk_ok
//				4 = data_ok
//				5 = extl_ok
//				6 = name_ok
//				7 = funk_glb_ok
//				8 = data_glb_ok
//				9 = extl_glb_ok
//	Value	: Neuer Wert
//	Ergibt den alten wert.
int LoadOptions(unsigned Item,int Value)
{
int		old;
int		pos;
int		bit;



	if(!Item)								// Alle setzen
		{
		old = 0;

		for(pos=0;pos<sizeof(options)/sizeof(options[0]);pos++)
			{
			bit  = LoadOptions(pos+1,(Value>>pos)&1);
			old	|= (bit&1)<<pos;
			}

		return old;
		}

	if(Item>sizeof(options)/sizeof(options[0]))
		{
		return 0;
		}

	old = *options[Item-1];
	*options[Item-1] = Value;


return old;
}
