//*****************************************************************************
//*
//*
//*		ObjLib.h
//*
//*
//*****************************************************************************
#ifndef 	__OBJLIB_H__
#define 	__OBJLIB_H__

#pragma pack(push,1)

#define 	OBJ_TYPE_OBJECT		0
#define 	OBJ_TYPE_SECTION	1
#define 	OBJ_TYPE_SYMBOL		2


typedef	struct
	{
	unsigned short	cpu_type;	// CPU-Type :
								//  0x014C = i386
	 							// 	0x014D = i486
								// 	0x014E = Pentium
								// 	0x0162 = MIPS R3000
								// 	0x0166 = MIPS R4000
								// 	0x0184 = DEC Alpha
								// 	0x01F0 = PowerPC
	unsigned short	sec_count;	// Anzahl der  Abschnitte in der Datei (=j)
	unsigned int	time_stamp;	// Zeitstempel (Sekunden seit 1.Jan.1970)
	unsigned int	sym_addr;	// Offset der zu Symboltabelle (bezogen auf den Header-Anfang)
	unsigned int	sym_count;	// Anzahl der Eintrge in der Symboltabelle (=m)
	unsigned short	opt_size;	// Gre des Optionalen-Headers (=n)
	unsigned short	flags;		// Flags
	}ObjHeader;

typedef	struct
	{
	unsigned int	virt_addr;	// Virtueller Offset
	unsigned int	sym_nr;		// Symbolnummer fr Realokation
	unsigned short	type;		//  6 = COFF_FIX_DIR32
								//  7 = COFF_FIX_RVA32
								// 20 = COFF_FIX_REL32

	}RealocEntry;


typedef	struct
	{
	union
		{
		char 		name[8];
		unsigned	offset[2];
		};

	unsigned int	value;	// Value
	unsigned short  sec_nr;	// Nummer des Abschnittes
	unsigned short  type;	// Typ
	unsigned char	art;	// Art des Eintrages:
							// 	 2 = Extern
							//   3 = Static
							//   6 = Label
							// 101 = Funktion
							// 103 = File
							// 104 = Sektion
							// 105 = Klasscall
	unsigned char	ext_nr;	// Anzahl der extra Eintrge am Ende (=i)
	}SymEntry;


typedef	struct
	{
	char			name[8];	// Name ohne Nullbyte
	unsigned int	virt_size;	// Virtuelle Gre
	unsigned int	virt_attr;	// Virtueller Offest
	unsigned int	raw_size;	// Abschnitts Gre
	unsigned int	raw_attr;	// Abschnitts Offest (bezogen auf den Header-Anfang)
	unsigned int	rel_addr;	// Reallokationen Offest (bezogen auf den Header-Anfang)
	unsigned int	line_addr;	// Zeilennummern  Offest (bezogen auf den Header-Anfang)
	unsigned short	rel_count;	// Anzahl der Reallokationen
	unsigned short	line_count;	// Anzahl der Zeilennummern
	unsigned int	flags;		// Flags:
	 							// 0x00000008 = No PAD
								// 0x00000020 = Enthlt Programmcode
								// 0x00000040 = Enthlt initialisierte Daten
								// 0x00000080 = Enthlt nicht initialisierte Daten
								// 0x00000200 = Enthlt Zusatzinfos (Kommentare)
								// 0x00000800 = Nicht im Speicher abbilten
								// 0x00001000 = Kompiniere Segmente
								// 0x00700000 = Align Flags (0=default,1=Byte,2=Word,3=DWord,4=8Byte,5=16Byte,6=32Byte,7=64Byte)
								// 0x01000000 = Enthlt Reallokationsdaten
								// 0x02000000 = Inhalt muss nicht gesichert werden
								// 0x04000000 = Inhalt nicht cachen
								// 0x08000000 = Inhalt nicht auslagern
								// 0x10000000 = Nicht von mehreren CPU's ausfhren
								// 0x20000000 = Ausfhrbar
								// 0x40000000 = Lesbar
								// 0x80000000 = Schreibgeschtzt
	}SecEntry;

typedef	struct
	{
	char type[16];				// "/         " fr Symbol-Listen
								// "//        " fr Datei-Listen
								// "/1234     " fr Objektdatei (die Zahl ist der Offset in der Datei-Liste)
	char timestamp[24];			// Zeitstempel als Text
	char value[8];				// Zahl als Text (Funktion ist unbekannt)
	char size[10];				// Gre des folgenden Blocks
	char id[2];					// 0x60 und 0x0A
	}LibHeader;

#pragma pack(pop)

	void LoadInit();
	int  LoadOptions(unsigned Item,int Value);
	int  LoadObj(const char *Ptr,unsigned int Size,unsigned Offset=0);
	int  LoadDll(const char *Ptr,unsigned int Size,unsigned Offset=0);
	int  LoadLib(const char *Ptr,unsigned int Size,unsigned Offset=0);
	int  LoadRel(const char *Ptr,unsigned int Size,unsigned *Data);

#endif
